/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef DPC_IMPL_TYPE_ROADINFO_H
#define DPC_IMPL_TYPE_ROADINFO_H

#include <cfloat>
#include <cmath>
#include "impl_type_uint64.h"
#include "dpc/impl_type_rmheader.h"
#include "dpc/impl_type_index.h"
#include "dpc/impl_type_carposition.h"
#include "dpc/impl_type_vector_lane.h"
#include "dpc/impl_type_vector_lanemarker.h"
#include "dpc/impl_type_vector_object.h"
#include "dpc/impl_type_vector_roadedge.h"
#include "dpc/impl_type_vector_passage.h"
#include "impl_type_float.h"
#include "impl_type_boolean.h"

namespace dpc {
struct RoadInfo {
    ::UInt64 fusion_timestamp{ 0U };
    ::dpc::RMHeader rm_header;
    ::dpc::Index road_id;
    ::dpc::CarPosition car_position;
    ::dpc::vector_Lane lane_list;
    ::dpc::vector_LaneMarker lane_marker_list;
    ::dpc::vector_Object object_list;
    ::dpc::vector_RoadEdge road_edge_list;
    ::dpc::vector_Passage passage_list;
    ::dpc::vector_Passage passage_list_ob;
    ::Float m_lastnode_s{ 0.0F };
    ::Float m_lastnode_vx{ 0.0F };
    ::Float m_lastnode_vy{ 0.0F };
    ::Float m_lastnode_ax{ 0.0F };
    ::Float m_lastnode_ay{ 0.0F };
    ::Float m_lastnode_jkx{ 0.0F };
    ::Float m_lastnode_spdlimt_val{ 0.0F };
    ::Float m_lastnode_spdlimt_dist{ 0.0F };
    ::Boolean m_lastnode_spdlimt_flag{ false };

    static bool IsPlane()
    {
        return false;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(fusion_timestamp);
        fun(rm_header);
        fun(road_id);
        fun(car_position);
        fun(lane_list);
        fun(lane_marker_list);
        fun(object_list);
        fun(road_edge_list);
        fun(passage_list);
        fun(passage_list_ob);
        fun(m_lastnode_s);
        fun(m_lastnode_vx);
        fun(m_lastnode_vy);
        fun(m_lastnode_ax);
        fun(m_lastnode_ay);
        fun(m_lastnode_jkx);
        fun(m_lastnode_spdlimt_val);
        fun(m_lastnode_spdlimt_dist);
        fun(m_lastnode_spdlimt_flag);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(fusion_timestamp);
        fun(rm_header);
        fun(road_id);
        fun(car_position);
        fun(lane_list);
        fun(lane_marker_list);
        fun(object_list);
        fun(road_edge_list);
        fun(passage_list);
        fun(passage_list_ob);
        fun(m_lastnode_s);
        fun(m_lastnode_vx);
        fun(m_lastnode_vy);
        fun(m_lastnode_ax);
        fun(m_lastnode_ay);
        fun(m_lastnode_jkx);
        fun(m_lastnode_spdlimt_val);
        fun(m_lastnode_spdlimt_dist);
        fun(m_lastnode_spdlimt_flag);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("fusion_timestamp", fusion_timestamp);
        fun("rm_header", rm_header);
        fun("road_id", road_id);
        fun("car_position", car_position);
        fun("lane_list", lane_list);
        fun("lane_marker_list", lane_marker_list);
        fun("object_list", object_list);
        fun("road_edge_list", road_edge_list);
        fun("passage_list", passage_list);
        fun("passage_list_ob", passage_list_ob);
        fun("m_lastnode_s", m_lastnode_s);
        fun("m_lastnode_vx", m_lastnode_vx);
        fun("m_lastnode_vy", m_lastnode_vy);
        fun("m_lastnode_ax", m_lastnode_ax);
        fun("m_lastnode_ay", m_lastnode_ay);
        fun("m_lastnode_jkx", m_lastnode_jkx);
        fun("m_lastnode_spdlimt_val", m_lastnode_spdlimt_val);
        fun("m_lastnode_spdlimt_dist", m_lastnode_spdlimt_dist);
        fun("m_lastnode_spdlimt_flag", m_lastnode_spdlimt_flag);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("fusion_timestamp", fusion_timestamp);
        fun("rm_header", rm_header);
        fun("road_id", road_id);
        fun("car_position", car_position);
        fun("lane_list", lane_list);
        fun("lane_marker_list", lane_marker_list);
        fun("object_list", object_list);
        fun("road_edge_list", road_edge_list);
        fun("passage_list", passage_list);
        fun("passage_list_ob", passage_list_ob);
        fun("m_lastnode_s", m_lastnode_s);
        fun("m_lastnode_vx", m_lastnode_vx);
        fun("m_lastnode_vy", m_lastnode_vy);
        fun("m_lastnode_ax", m_lastnode_ax);
        fun("m_lastnode_ay", m_lastnode_ay);
        fun("m_lastnode_jkx", m_lastnode_jkx);
        fun("m_lastnode_spdlimt_val", m_lastnode_spdlimt_val);
        fun("m_lastnode_spdlimt_dist", m_lastnode_spdlimt_dist);
        fun("m_lastnode_spdlimt_flag", m_lastnode_spdlimt_flag);
    }

    friend bool operator==(const ::dpc::RoadInfo& lhs, const ::dpc::RoadInfo& rhs) noexcept
    {
        return (lhs.fusion_timestamp == rhs.fusion_timestamp) && (lhs.rm_header == rhs.rm_header) && (lhs.road_id == rhs.road_id) && (lhs.car_position == rhs.car_position) && (lhs.lane_list == rhs.lane_list) && (lhs.lane_marker_list == rhs.lane_marker_list) && (lhs.object_list == rhs.object_list) && (lhs.road_edge_list == rhs.road_edge_list) && (lhs.passage_list == rhs.passage_list) && (lhs.passage_list_ob == rhs.passage_list_ob) && (fabs(static_cast<double>(lhs.m_lastnode_s - rhs.m_lastnode_s)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.m_lastnode_vx - rhs.m_lastnode_vx)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.m_lastnode_vy - rhs.m_lastnode_vy)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.m_lastnode_ax - rhs.m_lastnode_ax)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.m_lastnode_ay - rhs.m_lastnode_ay)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.m_lastnode_jkx - rhs.m_lastnode_jkx)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.m_lastnode_spdlimt_val - rhs.m_lastnode_spdlimt_val)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.m_lastnode_spdlimt_dist - rhs.m_lastnode_spdlimt_dist)) < DBL_EPSILON) && (lhs.m_lastnode_spdlimt_flag == rhs.m_lastnode_spdlimt_flag);
    }
};
} // namespace dpc


#endif // DPC_IMPL_TYPE_ROADINFO_H
